#!/usr/bin/env python3

import ntpath
import argparse
import os
from xml.etree.ElementTree import tostring

from ucca import convert
from ucca.ioutil import external_write_mode
from ucca.ioutil import get_passages_with_progress_bar
import xml.etree.ElementTree as ET

desc = """Removes <br> tokens from a standard XML."""


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    for fn in args.filenames:
        tree = ET.parse(fn)
        root = tree.getroot()
        to_remove = []
        old_to_new_ID = {}

        for node in root.getiterator():
            if node.tag == 'layer' and node.attrib.get('layerID',None) == "0":
                layer0 = node
                break

        last_parag = "1"
        position_in_paragraph = 0
        position = 1
        for node in layer0.getiterator():
            if node.tag == 'node':
                new_ID = '0.' + str(position)
                old_to_new_ID[node.attrib['ID']] = new_ID
                node.attrib['ID'] = new_ID
                for e in node.iter():
                    if e.tag == 'attributes':
                        if e.attrib.get('text',None) in ['','<br>']:
                            to_remove.append(node)
                        else:
                            position += 1
                            if e.attrib.get('paragraph', "0") != last_parag:
                                position_in_paragraph = 0
                                last_parag = e.attrib.get('paragraph', "0")
                            position_in_paragraph += 1
                            e.attrib['paragraph_position'] = str(position_in_paragraph)

        for node in to_remove:
            layer0.remove(node)

        # fixing layer1
        for node in root.getiterator():
            if node.tag == 'layer' and node.attrib.get('layerID',None) == "1":
                layer1 = node
                break

        for node in layer1.getiterator():
            if node.tag == 'edge':
                if node.attrib.get("toID",None) in old_to_new_ID.keys():
                    node.attrib["toID"] = old_to_new_ID[node.attrib["toID"]]

        P = convert.from_standard(root)
        xml_str = tostring(root).decode()
        site_filename = os.path.join(args.outdir,ntpath.basename(fn))
        f = open(site_filename,'w')
        f.write(xml_str)
        f.close()



if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+", help="XML file names to convert")
    argparser.add_argument("-o", "--outdir", default=".", help="output directory")
    argparser.add_argument("-v", "--verbose", action="store_true", help="verbose output")
    main(argparser.parse_args())



#!/usr/bin/env python3
import sys

import argparse
from tqdm import tqdm

from uccaapp.api import ServerAccessor

desc = """Sets the status of submitted tasks to ONGOING or SUBMITTED"""

ONGOING_STATUS = "ONGOING"
SUBMITTED_STATUS = "SUBMITTED"


class TaskStatusSetter(ServerAccessor):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def set_task_status(self, status, filename, **kwargs):
        del kwargs
        with open(filename) as f:
            task_ids = list(f.readlines())
        for task_id in task_ids:
            task = self.get_task(int(task_id))
            task["status"] = status
            task_out = self.update_task(**task)
            assert task_out["status"] == status
            yield task_out

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("filename", help="file with lines, each with a different task ID")
        argument_parser.add_argument("-s", "--status", help="should be ONGOING or SUBMITTED",
                                     choices=[ONGOING_STATUS, SUBMITTED_STATUS])
        ServerAccessor.add_arguments(argparser)


def main(**kwargs):
    list(TaskStatusSetter(**kwargs).set_task_status(**kwargs))


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    TaskStatusSetter.add_arguments(argument_parser)
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)



#!/usr/bin/env python3

import argparse
import sys
from collections import Counter, defaultdict

from ucca import layer1
from ucca.ioutil import get_passages_with_progress_bar

desc = """Parses XML files in UCCA standard format, and creates a histogram for the number of parents per unit."""


def plot_histogram(counter, label, plot=None):
    import matplotlib.pyplot as plt
    plt.figure()
    nums = list(counter.keys())
    counts = list(counter.values())
    indices = range(len(counts))
    bars = plt.bar(indices, counts, align="center")
    plt.xticks(indices, nums)
    top = 1.06 * max(counts)
    plt.ylim(min(counts), top)
    plt.xlabel("number of %s" % label)
    plt.ylabel("count")
    for bar in bars:
        count = bar.get_height()
        plt.text(bar.get_x() + bar.get_width() / 2., count, "%.1f%%" % (100.0 * count / sum(counts)),
                 ha="center", va="bottom")
    if plot:
        plt.savefig(plot + "histogram_" + label + ".png")
    else:
        plt.show()


def plot_pie(counter, label, plot=None):
    import matplotlib.pyplot as plt
    plt.figure()
    nums = list(counter.keys())
    counts = list(counter.values())
    plt.pie(counts, labels=nums, autopct="%1.1f%%",
            counterclock=True, wedgeprops={"edgecolor": "white"})
    plt.axis("equal")
    if plot:
        plt.savefig(plot + "pie_" + label + ".png")
    else:
        plt.show()


def main(args):
    histograms = defaultdict(Counter)
    for passage in get_passages_with_progress_bar(args.filenames):
        for node in passage.layer(layer1.LAYER_ID).all:
            if node.ID != "1.1":  # Exclude the root node
                histograms["parents"][clip(node.incoming, 3)] += 1
                histograms["children"][clip(node.outgoing, 7)] += 1

    for label, counter in histograms.items():
        handle = open(args.outfile + label + ".txt", "w", encoding="utf-8") if args.outfile else sys.stdout
        handle.writelines(["%s\t%d\n" % (num, count) for num, count in counter.items()])
        if handle is not sys.stdout:
            handle.close()
        try:
            plot_histogram(counter, label, plot=args.plot)
            plot_pie(counter, label, plot=args.plot)
        except:
            pass


def clip(l, m):
    return len(l) if len(l) <= m else ">%d" % m


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+", help="file names to analyze")
    argparser.add_argument("-o", "--outfile", default="data/counts_",
                           help="output file prefix for histogram")
    argparser.add_argument("-p", "--plot", default="data/plot_",
                           help="output file prefix for plot image file")
    main(argparser.parse_args())



#!/usr/bin/env python3
import argparse
import os
import sys

from ucca.ioutil import get_passages_with_progress_bar, write_passage

desc = """Rename passages by a given mapping of IDs"""


def main(filename, input_filenames, outdir):
    os.makedirs(outdir, exist_ok=True)
    with open(filename, encoding="utf-8") as f:
        pairs = [line.strip().split() for line in f]
        old_to_new_id = {old_id: new_id for new_id, old_id in pairs}
    for passage in get_passages_with_progress_bar(input_filenames, desc="Renaming"):
        passage._ID = old_to_new_id[passage.ID]
        write_passage(passage, outdir=outdir, verbose=False)


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    argument_parser.add_argument("filename", help="file with lines of the form <NEW ID> <OLD ID>")
    argument_parser.add_argument("input_filenames", help="filename pattern or directory with input passages")
    argument_parser.add_argument("-o", "--outdir", default=".", help="output directory")
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)



import pytest

from ucca import layer1
from ucca.normalization import normalize, COORDINATED_MAIN_REL
from .conftest import create_passage, attach_terminals

"""Tests normalization module correctness and API."""


def root_scene():
    p, l1, terms = create_passage()
    a1 = l1.add_fnode(None, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(None, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(None, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2)
    return p


def top_scene():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2)
    return p


def nested_center():
    p, l1, terms = create_passage(5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    f1 = l1.add_fnode(a1, layer1.EdgeTags.Function)
    c1 = l1.add_fnode(a1, layer1.EdgeTags.Center)
    f2 = l1.add_fnode(c1, layer1.EdgeTags.Function)
    c2 = l1.add_fnode(c1, layer1.EdgeTags.Center)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, f1, f2, c2, p1, a2)
    return p


def flat_center():
    p, l1, terms = create_passage(5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    f1 = l1.add_fnode(a1, layer1.EdgeTags.Function)
    f2 = l1.add_fnode(a1, layer1.EdgeTags.Function)
    c1 = l1.add_fnode(a1, layer1.EdgeTags.Center)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, f1, f2, c1, p1, a2)
    return p


def unary_center():
    p, l1, terms = create_passage(5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    c1 = l1.add_fnode(a1, layer1.EdgeTags.Center)
    f1 = l1.add_fnode(c1, layer1.EdgeTags.Function)
    f2 = l1.add_fnode(c1, layer1.EdgeTags.Function)
    c2 = l1.add_fnode(c1, layer1.EdgeTags.Center)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, f1, f2, c2, p1, a2)
    return p


def unary_function():
    p, l1, terms = create_passage(5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    f1 = l1.add_fnode(a1, layer1.EdgeTags.Function)
    f2 = l1.add_fnode(f1, layer1.EdgeTags.Function)
    c1 = l1.add_fnode(f1, layer1.EdgeTags.Center)
    c2 = l1.add_fnode(a1, layer1.EdgeTags.Center)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, f2, c1, c2, p1, a2)
    return p


def unary_participant():
    """
    Whenever there is an A as an only child, remove it.
    """
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    ps2 = l1.add_fnode(a, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    linker = l1.add_fnode(a, layer1.EdgeTags.Linker)
    ps3 = l1.add_fnode(a, layer1.EdgeTags.ParallelScene)
    p2 = l1.add_fnode(ps3, layer1.EdgeTags.Process)
    attach_terminals(terms, p1, linker, p2)
    return p


def simple_linkage():
    """
    Whenever there is an A as an only child, remove it.
    """
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    linker = l1.add_fnode(None, layer1.EdgeTags.Linker)
    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    p2 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    attach_terminals(terms, p1, linker, p2)
    return p


def simple_function():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    f1 = l1.add_fnode(ps1, layer1.EdgeTags.Function)
    attach_terminals(terms, a1, p1, f1)
    return p


def complex_function():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    f1 = l1.add_fnode(ps1, layer1.EdgeTags.Function)
    p1 = l1.add_fnode(f1, layer1.EdgeTags.Process)
    c2 = l1.add_fnode(f1, layer1.EdgeTags.Center)
    attach_terminals(terms, a1, p1, c2)
    return p


def unary_punct():
    p, l1, terms = create_passage(3, 3)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1)
    l1.add_punct(a2, terms[2])
    return p


def unattached_punct():
    p, l1, terms = create_passage(3, 3)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    return p


def top_punct():
    p, l1, terms = create_passage(3, 3)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    l1.add_punct(None, terms[2])
    return p


def attached_punct():
    p, l1, terms = create_passage(3, 3)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    l1.add_punct(ps1, terms[2])
    return p


def top_punct_only():
    p, l1, terms = create_passage(1, 1)
    l1.add_punct(None, terms[0])
    return p


def moved_punct():
    p, l1, terms = create_passage(3, 3)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    l1.add_punct(a1, terms[2])
    return p


def multi_punct():
    p, l1, terms = create_passage(4, 3, 4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    punct1 = l1.add_punct(ps1, terms[2])
    punct1.add(layer1.EdgeTags.Terminal, terms[3])
    return p


def split_punct():
    p, l1, terms = create_passage(4, 3, 4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    l1.add_punct(ps1, terms[2])
    l1.add_punct(ps1, terms[3])
    return p


def cycle():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    l1.add_remote(a2, layer1.EdgeTags.Elaborator, ps1)
    attach_terminals(terms, a1, p1, a2)
    return p


def unanalyzable():
    p, l1, terms = create_passage(5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2, a2, a2)
    return p


def unanalyzable_punct():
    p, l1, terms = create_passage(5, 3, 4, 5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2, a2, a2)
    return p


def punct_inside_unanalyzable():
    p, l1, terms = create_passage(5, 4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2)
    a2.add(layer1.EdgeTags.Terminal, terms[4])
    l1.add_punct(a2, terms[3])
    return p


def unattached_punct_inside_unanalyzable():
    p, l1, terms = create_passage(5, 4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2)
    a2.add(layer1.EdgeTags.Terminal, terms[4])
    return p


def punct_outside_unanalyzable():
    p, l1, terms = create_passage(5, 5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2, a2)
    l1.add_punct(ps1, terms[4])
    return p


def unattached_punct_outside_unanalyzable():
    p, l1, terms = create_passage(5, 5)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    attach_terminals(terms, a1, p1, a2, a2)
    return p


def unattached_terms():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    attach_terminals(terms, a1, p1)
    return p


def attached_terms():
    p, l1, terms = create_passage()
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    f1 = l1.add_fnode(ps1, layer1.EdgeTags.Function)
    attach_terminals(terms, a1, p1, f1)
    return p


def cmr_no_attrib():
    p, l1, terms = create_passage(4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    c1 = l1.add_fnode(p1, layer1.EdgeTags.Center)
    n = l1.add_fnode(p1, layer1.EdgeTags.Connector)
    c2 = l1.add_fnode(p1, layer1.EdgeTags.Center)
    attach_terminals(terms, a, c1, n, c2)
    return p


def insert_cmr(l1, scene):
    p = l1.add_fnode(scene, layer1.EdgeTags.Process)
    p.incoming[0].attrib[COORDINATED_MAIN_REL] = True
    c1 = l1.add_fnode(p, layer1.EdgeTags.Center)
    n = l1.add_fnode(p, layer1.EdgeTags.Connector)
    c2 = l1.add_fnode(p, layer1.EdgeTags.Center)
    return p, c1, n, c2


def cmr():
    p, l1, terms = create_passage(4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1, c1, n, c2 = insert_cmr(l1, ps1)
    attach_terminals(terms, a, c1, n, c2)
    return p


def expanded_cmr():
    p, l1, terms = create_passage(4)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    linker = l1.add_fnode(None, layer1.EdgeTags.Linker)
    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    l1.add_remote(ps2, layer1.EdgeTags.Participant, a)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    p2 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    attach_terminals(terms, a, p1, linker, p2)
    return p


def remote_cmr():
    p, l1, terms = create_passage(6)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    linker = l1.add_fnode(None, layer1.EdgeTags.Linker)
    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1, c1, n, c2 = insert_cmr(l1, ps1)
    a2 = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
    l1.add_remote(ps2, layer1.EdgeTags.Process, p1)
    attach_terminals(terms, a1, c1, n, c2, linker, a2)
    return p


def expanded_remote_cmr():
    p, l1, terms = create_passage(6)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    linker1 = l1.add_fnode(None, layer1.EdgeTags.Linker)
    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    linker2 = l1.add_fnode(None, layer1.EdgeTags.Linker)
    ps3 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    # ps4 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)  # ??? scene with only remotes
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    l1.add_remote(ps2, layer1.EdgeTags.Participant, a1)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    p2 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps3, layer1.EdgeTags.Participant)
    l1.add_remote(ps3, layer1.EdgeTags.Process, p1)
    # l1.add_remote(ps4, layer1.EdgeTags.Process, p2)
    # l1.add_remote(ps4, layer1.EdgeTags.Participant, a2)  # ???
    attach_terminals(terms, a1, p1, linker1, p2, linker2, a2)
    return p


def cmr_with_relator_outside():
    """
    [suitable_S [for_R the_F [breeding_C and_N growing_C]_P|CMR [of_R giants_C]_A]_A]_H
    """
    p, l1, terms = create_passage(8)
    ps = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    s = l1.add_fnode(ps, layer1.EdgeTags.State)
    a_scene = l1.add_fnode(ps, layer1.EdgeTags.Participant)
    r1 = l1.add_fnode(a_scene, layer1.EdgeTags.Relator)
    f = l1.add_fnode(a_scene, layer1.EdgeTags.Function)
    p1, c1, n, c2 = insert_cmr(l1, a_scene)
    a2 = l1.add_fnode(a_scene, layer1.EdgeTags.Participant)
    r2 = l1.add_fnode(a2, layer1.EdgeTags.Relator)
    c3 = l1.add_fnode(a2, layer1.EdgeTags.Center)
    attach_terminals(terms, s, r1, f, c1, n, c2, r2, c3)
    return p


def expanded_cmr_with_relator_outside():
    """
    [suitable_S [for_R [[the_F breeding_C]_P [of_R giants_C]_A]_H and_L [[growing_P] [*of giants]_A]_H]_H
    """
    p, l1, terms = create_passage(8)
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    s = l1.add_fnode(ps1, layer1.EdgeTags.State)
    a_superscene = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    r1 = l1.add_fnode(a_superscene, layer1.EdgeTags.Relator)
    ps2 = l1.add_fnode(a_superscene, layer1.EdgeTags.ParallelScene)
    linker = l1.add_fnode(a_superscene, layer1.EdgeTags.Linker)
    ps3 = l1.add_fnode(a_superscene, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    f = l1.add_fnode(ps2, layer1.EdgeTags.Function)
    a = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
    r2 = l1.add_fnode(a, layer1.EdgeTags.Relator)
    c2 = l1.add_fnode(a, layer1.EdgeTags.Center)
    p2 = l1.add_fnode(ps3, layer1.EdgeTags.Process)
    l1.add_remote(ps3, layer1.EdgeTags.Participant, a)
    attach_terminals(terms, s, r1, f, p1, linker, p2, r2, c2)
    return p


def normalize_and_compare(unnormalized, normalized, extra=False):
    p1 = unnormalized()
    p2 = normalized()
    if unnormalized != normalized:
        assert not p1.equals(p2), "Unnormalized and normalized passage: %s == %s" % (str(p1), str(p2))
    normalize(p1, extra=extra)
    assert p1.equals(p2), "Normalized passage: %s != %s" % (str(p1), str(p2))


@pytest.mark.parametrize("unnormalized, normalized", (
        (root_scene, top_scene),
        (nested_center, flat_center),
        (unary_center, flat_center),
        (unary_function, flat_center),
        (complex_function, simple_function),
        (unary_punct, attached_punct),
        (unattached_punct, attached_punct),
        (top_punct, attached_punct),
        (top_punct_only, top_punct_only),
        (moved_punct, attached_punct),
        (multi_punct, split_punct),
        (cycle, top_scene),
        (unanalyzable, unanalyzable),
        (unanalyzable_punct, unanalyzable_punct),
        (punct_inside_unanalyzable, punct_inside_unanalyzable),
        (unattached_punct_inside_unanalyzable, punct_inside_unanalyzable),
        (punct_outside_unanalyzable, punct_outside_unanalyzable),
        (unattached_punct_outside_unanalyzable, punct_outside_unanalyzable),
        (cmr_no_attrib, cmr_no_attrib),
        (cmr, expanded_cmr),
        (remote_cmr, expanded_remote_cmr),
        (cmr_with_relator_outside, expanded_cmr_with_relator_outside),
        (unary_participant, simple_linkage),
))
def test_normalize(unnormalized, normalized):
    normalize_and_compare(unnormalized, normalized)


@pytest.mark.parametrize("unnormalized, normalized", (
        (unattached_terms, attached_terms),
))
def test_normalize_extra(unnormalized, normalized):
    normalize_and_compare(unnormalized, normalized, extra=True)



import string
from itertools import groupby
from operator import attrgetter

from ucca import layer0, layer1
from ucca.layer0 import NodeTags as L0Tags
from ucca.layer1 import EdgeTags as ETags, NodeTags as L1Tags
from logging import warning

LINKAGE = {ETags.LinkArgument, ETags.LinkRelation}
NON_SCENE = {ETags.Center, ETags.Elaborator, ETags.Quantifier, ETags.Connector}
SUPP_FUNC = {ETags.Relator, ETags.Function, ETags.Unanalyzable, ETags.Uncertain}
SCENE = {ETags.Participant, ETags.State, ETags.Process, ETags.Adverbial, ETags.Time}

def validate(passage, linkage=True, multigraph=False):
    for node in passage.layer(layer0.LAYER_ID).all:
        yield from NodeValidator(node).validate_terminal()
    heads = list(passage.layer(layer1.LAYER_ID).heads)
    found_linkage = False
    for node in heads:
        if node.tag == L1Tags.Linkage:
            found_linkage = True
        yield from NodeValidator(node).validate_top_level()
    stack = [heads]
    visited = set()
    path = []
    path_set = set(path)
    while stack:
        for node in stack[-1]:
            if node in path_set:
                yield "Detected cycle (%s)" % "->".join(n.ID for n in path)
            elif node not in visited:
                visited.add(node)
                path.append(node)
                path_set.add(node)
                stack.append(node.children)
                yield from NodeValidator(node).validate_non_terminal(linkage=linkage and found_linkage,
                                                                     multigraph=multigraph)
                break
        else:
            if path:
                path_set.remove(path.pop())
            stack.pop()


class NodeValidator:
    def __init__(self, node):
        self.node = node
        self.node_id = self.node.ID
        tree_id = self.node.extra.get("tree_id")
        if tree_id:
            self.node_id += ", %s" % tree_id
        self.incoming = tag_to_edge(node.incoming)
        self.outgoing = tag_to_edge(node)
        self.incoming_tags = set(self.incoming)
        self.outgoing_tags = set(self.outgoing)

    def validate_terminal(self):
        if not self.node.text:
            yield "Empty terminal text (%s)" % self.node_id
        if set(self.node.text).intersection(string.whitespace):
            yield "Whitespace in terminal text (%s): '%s'" % (self.node_id, self.node)
        if not self.incoming:
            yield "Orphan %s terminal (%s) '%s'" % (self.node.tag, self.node_id, self.node)
        elif len(self.node.incoming) > 1:
            yield "Reentrant %s terminal (%s) '%s'" % (self.node.tag, join(self.node.incoming), self.node)

    def validate_top_level(self):
        if self.node not in self.node.layer.heads and self.node.tag != L1Tags.Linkage:
            yield "Extra root (%s)" % self.node_id
        terminals = [n for n in self.node.children if n.layer.ID == layer0.LAYER_ID]
        if terminals:
            yield "Terminal children (%s) of root (%s)" % (join(terminals), self.node_id)
        s = self.outgoing_tags.difference((ETags.ParallelScene, ETags.Linker, ETags.Function, ETags.Ground,
                                           ETags.Punctuation, ETags.LinkRelation, ETags.LinkArgument))
        if s:
            yield "Top-level unit (%s) with %s children: %s" %\
                  (self.node_id, join(s), join(e.child for e in self.node if s.intersection(e.tags)))

    def validate_non_terminal(self, linkage=False, multigraph=False):
        if linkage and self.node.tag == L1Tags.Linkage:
            yield from self.validate_linkage()
        elif self.node.tag == L1Tags.Foundational:
            yield from self.validate_foundational()
        primary_incoming = [e for e in self.node.incoming if not e.attrib.get("remote") and
                            not LINKAGE.intersection(e.tags)]
        if len(primary_incoming) > 1:
            yield "Unit (%s) with multiple non-remote parents (%s)" % (self.node_id, join(primary_incoming))
        remote_incoming = [e for e in self.node.incoming if e.attrib.get("remote")]
        if remote_incoming and not primary_incoming:
            yield "Unit (%s) with remote parents but no primary parents" % self.node_id
        for edge in self.node:
            if (ETags.Punctuation in edge.tags) != (edge.child.tag == L1Tags.Punctuation):
                yield "%s edge (%s) with %s child" % (edge.tags, edge, edge.child.tag)
            # FN parent of Punctuation is disallowed unless the FN is unanalyzable
            if (self.node.tag == L1Tags.Foundational) and (edge.child.tag == L0Tags.Punct) and \
                    not len(self.node.terminals) + len(self.node.punctuation) == len(self.node.children) > 1 or \
                    (self.node.tag == L1Tags.Punctuation) and not (edge.child.tag == L0Tags.Punct):
                yield "%s unit (%s) with %s child (%s)" % (self.node.tag, self.node_id, edge.child.tag, edge.child.ID)
        if self.node.attrib.get("implicit"):
            if self.node.outgoing:
                yield "Implicit unit (%s) with children (%s)" % (self.node_id, join(self.node.children))
        elif self.node.tag in (L1Tags.Foundational, L1Tags.Linkage, L1Tags.Punctuation) and \
                all(e.attrib.get("remote") for e in self.node):
            yield "Non-implicit unit (%s) with no primary children" % (self.node_id)
        for tag in (ETags.Function, ETags.LinkRelation,
                    ETags.Connector, ETags.Punctuation, ETags.Terminal):
            s = self.incoming.get(tag, ())
            if len(s) > 1:
                yield "Unit (%s) with multiple %s parents (%s)" % (self.node_id, tag, join(e.parent for e in s))
        for tag in (ETags.LinkRelation, ETags.Process, ETags.State):
            s = self.outgoing.get(tag, ())
            if len(s) > 1:
                yield "Unit (%s) with multiple %s children (%s)" % (self.node_id, tag, join(e.child for e in s))
        s = self.outgoing_tags.difference(set.union({ETags.Terminal, ETags.Punctuation}, NON_SCENE, SUPP_FUNC))
        if (ETags.Function in self.incoming) and s:
            yield "%s unit (%s) with %s children: %s" % (ETags.Function, self.node_id, join(s), self.node)
        if ETags.Linker in self.incoming_tags and linkage and ETags.LinkRelation not in self.incoming_tags:
            yield "%s unit (%s) with no incoming %s" % (ETags.Linker, self.node_id, ETags.LinkRelation)
        if not multigraph:
            key = attrgetter("child.ID")
            for child_id, edges in groupby(sorted(self.node, key=key), key=key):
                edges = list(edges)
                if len(edges) > 1:
                    yield "Multiple edges from %s to %s: %s" % (self.node_id, child_id, ", ".join(
                        "%d %s" % (len(e), t) for t, e in tag_to_edge(edges).items()))
        no_sub_non_scene = (ETags.Process, ETags.Adverbial, ETags.Linker, ETags.Time, ETags.Quantifier,
                            ETags.Connector, ETags.State)
        s = self.outgoing_tags.difference((set.union({ETags.Terminal, ETags.Punctuation}, NON_SCENE, SUPP_FUNC)))
        if any(self.incoming_tags.intersection(no_sub_non_scene)) and s:
            edges_to_check =  self.incoming_tags.intersection(no_sub_non_scene)
            yield "%s unit (%s) with %s children: %s" % (join(edges_to_check), self.node_id, join(s), self.node)
        if (ETags.Unanalyzable in self.incoming) and self.outgoing:
            yield "%s unit (%s) with children: %s" % (ETags.Unanalyzable, self.node_id, self.node) # FIXME: should include punctuation and terminal?
        s = self.outgoing_tags.intersection({ETags.ParallelScene, ETags.Linker, ETags.Ground})
        if (ETags.Ground in self.incoming) and s:
            yield "%s unit (%s) with %s children: %s" % (ETags.Ground, self.node_id, join(s), self.node)
        s = self.outgoing_tags.intersection({ETags.ParallelScene, ETags.Linker})
        if (ETags.ParallelScene in self.incoming) and s:
            yield "%s unit (%s) with %s children: %s" % (ETags.ParallelScene, self.node_id, join(s), self.node)
        forbidden = {ETags.ParallelScene, ETags.Linker, ETags.Process, ETags.Center, ETags.Elaborator,
                     ETags.Quantifier, ETags.Connector}
        s = forbidden.intersection(self.outgoing_tags)
        if (ETags.State in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.State, join(s), self.node)
        s = self.outgoing_tags.intersection({ETags.ParallelScene, ETags.Linker})
        if (ETags.Adverbial in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Adverbial, join(s), self.node)
        s = self.outgoing_tags.difference((set.union({ETags.Terminal, ETags.Punctuation}, {ETags.Adverbial},
                                                     NON_SCENE, SUPP_FUNC)))
        if (ETags.Center in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Center, join(s), self.node)
        forbidden = set.union(NON_SCENE, ETags.ParallelScene, ETags.Linker)
        s = forbidden.intersection(self.outgoing_tags)
        if (any(self.outgoing_tags.intersection({ETags.Time, ETags.Participant}))) and s:
            edges_to_check = [k for k in self.outgoing_tags if k in (ETags.Time, ETags.Participant)]
            yield "%s unit with %s siblings: under %s" % (join(edges_to_check), join(s), self.node)
        forbidden = set.union(NON_SCENE, SCENE)
        s = self.outgoing_tags.intersection(forbidden)
        if (any(self.outgoing_tags.intersection({ETags.ParallelScene, ETags.Linker}))) and s:
            edges_to_check = self.outgoing_tags.intersection({ETags.ParallelScene, ETags.Linker})
            yield "%s unit with %s siblings: under %s" % (join(edges_to_check), join(s), self.node)
        forbidden = set.union(SCENE, ETags.ParallelScene, ETags.Linker, ETags.Ground, ETags.Connector)
        s = forbidden.intersection(self.outgoing_tags)
        if 'D' in s:
            s.remove('D')
        if (ETags.Elaborator in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Elaborator, join(s), self.node)
        forbidden = set.union(SCENE, ETags.ParallelScene, ETags.Linker, ETags.Ground)
        s = forbidden.intersection(self.outgoing_tags)
        if 'D' in s:
            s.remove('D')
        if (ETags.Quantifier in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Quantifier, join(s), self.node)
        if (any(self.outgoing_tags.intersection({ETags.Quantifier, ETags.Elaborator}))) and (ETags.Adverbial in self.outgoing_tags):
            warning("Adverbial with Quantifier and/or Elaborator under $s" % (self.node))
        forbidden = set.union(SCENE, ETags.ParallelScene, ETags.Linker, ETags.Ground, ETags.Elaborator,
                              ETags.Quantifier)
        s = forbidden.intersection(self.outgoing_tags)
        if (ETags.Connector in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Connector, join(s), self.node)
        forbidden = set.union(NON_SCENE, ETags.State)
        s = forbidden.intersection(self.outgoing_tags)
        if (ETags.Process in self.outgoing_tags) and s:
            yield "%s unit with %s siblings: under %s" % (ETags.Process, join(s), self.node)
        if any(self.outgoing_tags.intersection({ETags.Elaborator, ETags.Quantifier, ETags.Connector})):
            edges_to_check = list(self.outgoing_tags.intersection({ETags.Elaborator, ETags.Quantifier, ETags.Connector}))
            if ETags.Center not in self.outgoing_tags:
                yield "%s unit without %s sibling: under %s" % (join(edges_to_check), ETags.Center, self.node)
        if any(self.outgoing_tags.intersection({ETags.Time, ETags.Participant})):
            edges_to_check = list(self.outgoing_tags.intersection({ETags.Time, ETags.Participant}))
            required = {ETags.State, ETags.Process}
            if ETags.State not in self.outgoing_tags and ETags.Process not in self.outgoing_tags:
                yield "%s unit without %s siblings: under %s" % (join(edges_to_check), join(required), self.node)
        if (ETags.Linker in self.outgoing_tags) and (ETags.ParallelScene not in self.outgoing_tags):
            yield "%s unit without %s sibling: under %s" % (ETags.Linker, ETags.ParallelScene, self.node)
        if any(self.incoming_tags.intersection({ETags.State, ETags.Process})):
            edges_to_check = list(self.incoming_tags.intersection({ETags.State, ETags.Process}))
            forbidden = set.union(SCENE, ETags.ParallelScene, ETags.Linker, ETags.Ground)
            itr = self.node.iter()
            for i in itr:
                if not isinstance(i, layer0.Terminal):
                    if i.parallel_scenes or i.linkers or i.grounds or i.participants or i.state or i.process or i.adverbials or i.times:
                        yield "%s unit (%s) with at least one of the %s descendants: %s" % (join(edges_to_check), self.node_id, join(forbidden), self.node)
        s = [e for e in self.node.incoming if
             e.attrib.get('remote') and e.tag == ETags.Function]
        if (ETags.Function in self.incoming_tags) and s:
            yield "%s remote edges (%s)" % (ETags.Function, join(s))
        s = [e for e in self.node.incoming if
             e.attrib.get('remote') and e.tag == ETags.Relator]
        if (ETags.Relator in self.incoming_tags) and s:
            warning("Relator remote edges (%s)" % (join(s)))
        s = self.outgoing_tags.difference(set.union({ETags.ParallelScene, ETags.Linker, ETags.Function,
                                                     ETags.Punctuation}, LINKAGE))
        if (not self.incoming) and s:
            yield "%s unit (%s) at top level" % (join(s), self.node_id)
        s = [e for e in self.node.incoming if ETags.Unanalyzable in e.tags and len(set(e.tags)) == 1]
        if (ETags.Unanalyzable in self.incoming_tags) and s:
            yield "%s unit (%s) without another label" % (ETags.Unanalyzable, self.node_id)

    def validate_linkage(self):
        if self.node.incoming:
            yield "Non-root %s unit (%s)" % (self.node.tag, self.node_id)
        s = self.outgoing_tags.difference(LINKAGE)
        if s:
            yield "%s unit (%s) with %s children" % (self.node.tag, self.node_id, join(s))
        if ETags.LinkRelation not in self.outgoing:
            yield "%s unit without %s child" % (self.node.tag, ETags.LinkRelation)

    def validate_foundational(self):
        if self.node.participants and not self.node.is_scene():
            yield "Unit (%s) with participants but without main relation: %s" % (self.node_id, self.node)
        if self.node.process and self.node.state:
            yield "Unit (%s) with both process (%s) and state (%s)" % (self.node_id, self.node.process, self.node.state)
        if self.node.parallel_scenes:
            s = self.outgoing_tags.difference((ETags.ParallelScene, ETags.Punctuation, ETags.Linker,
                                               ETags.Ground, ETags.Relator, ETags.Function))
            if s:
                yield "Unit (%s) with parallel scenes has %s children: %s" %\
                      (self.node_id, join(s), join(e.child for e in self.node if s.intersection(e.tags)))
        s = self.outgoing_tags.intersection(LINKAGE)
        if s:
            yield "Non-linkage unit (%s) with %s children: %s" %\
                  (self.node_id, join(s), join(e.child for e in self.node if s.intersection(e.tags)))



def tag_to_edge(edges):
    d = {}
    for edge in edges:
        for tag in edge.tags:
            d.setdefault(tag, []).append(edge)
    return d


def join(items):
    return ", ".join(map(str, items))



#!/usr/bin/env python3
import sys

import argparse
import os
from tqdm import tqdm

from ucca.ioutil import file2passage, passage2file, external_write_mode

desc = """Parses an XML in UCCA standard format, and writes them in binary Pickle format."""


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    for filename in tqdm(args.filenames, desc="Converting", unit=" passages"):
        if args.verbose:
            with external_write_mode():
                print("Reading passage '%s'..." % filename, file=sys.stderr)
        passage = file2passage(filename)
        basename = os.path.splitext(os.path.basename(filename))[0]
        outfile = args.outdir + os.path.sep + basename + ".pickle"
        if args.verbose:
            with external_write_mode():
                print("Writing file '%s'..." % outfile, file=sys.stderr)
        passage2file(passage, outfile, binary=True)


if __name__ == '__main__':
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument('filenames', nargs='+', help="XML file names to convert")
    argparser.add_argument('-o', '--outdir', default='.', help="output directory")
    argparser.add_argument('-v', '--verbose', action="store_true", help="verbose output")
    main(argparser.parse_args())



#!/usr/bin/env python3

import argparse
import glob
import os
import sys

desc = """Combines several SDP parsed files to one.
"""


def main(args):
    lines = [args.prefix + args.header + "\n"]
    for pattern in args.filenames:
        filenames = sorted(glob.glob(pattern))
        if not filenames:
            raise IOError("Not found: " + pattern)
        for filename in filenames:
            base = os.path.basename(os.path.splitext(filename)[0])
            lines.append(args.prefix + base + "\n")
            with open(filename, encoding="utf-8") as f:
                lines += f.readlines()
        f = sys.stdout if args.outfile is None else open(args.outfile, "w", encoding="utf-8")
        f.writelines(lines)
        if args.outfile is not None:
            f.close()


if __name__ == '__main__':
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+",
                           help="SDP file names to join")
    argparser.add_argument("-o", "--outfile",
                           help="output filename (standard output if unspecified)")
    argparser.add_argument("-H", "--header", default="SDP 2015",
                           help="first line in the file, not including prefix")
    argparser.add_argument("-p", "--prefix", default="#",
                           help="prefix for comment lines")
    main(argparser.parse_args())



import re
import warnings
from collections import defaultdict
from operator import attrgetter

from ucca import layer0, layer1


def node_label(node):
    return re.sub("[^(]*\((.*)\)", "\\1", node.attrib.get("label", ""))


def draw(passage, node_ids=False):
    import matplotlib.cbook
    import networkx as nx
    warnings.filterwarnings("ignore", category=matplotlib.cbook.mplDeprecation)
    warnings.filterwarnings("ignore", category=UserWarning)
    g = nx.DiGraph()
    g.add_nodes_from([(n.ID, {"label": n.text, "color": "white"}) for n in passage.layer(layer0.LAYER_ID).all])
    g.add_nodes_from([(n.ID, {"label": "IMPLICIT", "color": "white"}) for n in passage.layer(layer1.LAYER_ID).all
                      if n.attrib.get("implicit")])
    g.add_nodes_from([(n.ID, {"label": node_label(n) or (n.ID if node_ids else ""),
                              "color": "gray" if n.tag == layer1.NodeTags.Linkage else
                              ("white" if node_label(n) or (n.ID and node_ids) else "black")})
                      for n in passage.layer(layer1.LAYER_ID).all if not n.attrib.get("implicit")])
    g.add_edges_from([(n.ID, e.child.ID, {"label": "|".join(e.tags),
                                          "style": "dashed" if e.attrib.get("remote") else "solid"})
                      for layer in passage.layers for n in layer.all for e in n])
    pos = topological_layout(passage)
    nx.draw(g, pos, arrows=False, font_size=10,
            node_color=[d["color"] for _, d in g.nodes(data=True)],
            labels={n: d["label"] for n, d in g.nodes(data=True) if d["label"]},
            style=[d["style"] for _, _, d in g.edges(data=True)])
    nx.draw_networkx_edge_labels(g, pos, font_size=8,
                                 edge_labels={(u, v): d["label"] for u, v, d in g.edges(data=True)})


def topological_layout(passage):
    visited = defaultdict(set)
    pos = {}
    terminals = passage.layer(layer0.LAYER_ID).all
    if terminals:
        implicit_offset = list(range(0, 1 + max(n.position for n in terminals)))
        leaves = sorted([n for layer in passage.layers for n in layer.all if not n.children],
                        key=lambda n: getattr(n, "position", None) or (n.fparent.end_position if n.fparent else -1))
        for node in leaves:  # draw leaves first to establish ordering
            if node.layer.ID == layer0.LAYER_ID:  # terminal
                x = node.position
                pos[node.ID] = (x + sum(implicit_offset[:x + 1]), 0)
            elif node.fparent:  # implicit
                implicit_offset[node.fparent.end_position] += 1
    else:
        implicit_offset = [0]
    remaining = [n for n in passage.layer(layer1.LAYER_ID).all if not n.parents]
    implicits = []
    while remaining:  # draw non-terminals
        node = remaining.pop()
        if node.ID in pos:  # done already
            continue
        children = [c for c in node.children if c.ID not in pos and c not in visited[node.ID]]
        if children:
            visited[node.ID].update(children)  # to avoid cycles
            remaining += [node] + children
            continue
        if node.children:
            xs, ys = zip(*(pos[c.ID] for c in node.children if not c.attrib.get("implicit")))
            pos[node.ID] = sum(xs) / len(xs), 1 + max(ys)  # done with children
        else:
            implicits.append(node)
    for node in implicits:
        fparent = node.fparent or passage.layer(layer1.LAYER_ID).heads[0]
        x = fparent.end_position
        x += sum(implicit_offset[:x + 1])
        _, y = pos.get(fparent.ID, (0, 0))
        pos[node.ID] = (x, y - 1)
    pos = {i: (x, y ** 1.01)for i, (x, y) in pos.items()}  # stretch up to avoid over cluttering
    return pos


TEX_ESCAPE_TABLE = {
    "&": r"\&",
    "%": r"\%",
    "$": r"\$",
    "#": r"\#",
    "_": r"\_",
    "{": r"\{",
    "}": r"\}",
    "~": r"\textasciitilde{}",
    "^": r"\^{}",
    "\\": r"\textbackslash{}",
    "<": r"\textless ",
    ">": r"\textgreater ",
}
TEX_ESCAPE_PATTERN = re.compile("|".join(map(re.escape, sorted(TEX_ESCAPE_TABLE, key=len, reverse=True))))


def tex_escape(text):
    """
        :param text: a plain text message
        :return: the message escaped to appear correctly in LaTeX
    """
    return TEX_ESCAPE_PATTERN.sub(lambda match: TEX_ESCAPE_TABLE[match.group()], text)


def tikz(p, indent=None, node_ids=False):
    """
    Visualize to TikZ format
    :param p: Passage
    :param indent: indentation size or None for no indentation
    :param node_ids: whether to include node IDs
    :return: string in TikZ format
    """
    if indent is None:
        l1 = p.layer(layer1.LAYER_ID)
        return r"""
\begin{tikzpicture}[->,level distance=1cm,
  level 1/.style={sibling distance=4cm},
  level 2/.style={sibling distance=15mm},
  level 3/.style={sibling distance=15mm},
  every circle node/.append style={%s=black},
  every node/.append style={text height=.6ex,text depth=0}]
  \tikzstyle{word} = [font=\rmfamily,color=black]
  """ % ("draw" if node_ids else "fill") + "\\" + tikz(l1.heads[0], indent=1, node_ids=node_ids) + \
            "\n".join([";"] + [r"  \draw[dashed,->] (%s) to node [auto] {%s} (%s);" %
                               (e.parent.ID.replace(".", "_"), "|".join(e.tags), e.child.ID.replace(".", "_"))
                               for n in l1.all for e in n if e.attrib.get("remote")] + [r"\end{tikzpicture}"])
    return "node (" + p.ID.replace(".", "_") + ") " + (
        ("[word] {" +
         (" ".join(tex_escape(t.text)
                   for t in sorted(p.terminals, key=attrgetter("position"))) or r"\textbf{IMPLICIT}")
         + "} ") if p.terminals or p.attrib.get("implicit") else ("\n" + indent * "  ").join(
            ["[circle] {%s}" % (node_label(p) or (p.ID if node_ids else "")), "{"] +
            ["child {" + tikz(e.child, indent + 1) +
             " edge from parent node[auto]  {" + "|".join(e.tags) + "}}"
             for e in sorted(p, key=lambda f: f.child.start_position)
             if not e.attrib.get("remote")] +
            ["}"]))


def standoff(p):
    """
    Visualize to Standoff .ann format, which can be presented with brat
    :param p: Passage
    :return: string in Standoff format
    """
    l0 = p.layer(layer0.LAYER_ID)
    terminal_start = {}
    terminal_end = {}
    start = end = 0
    for terminal in sorted(l0.all, key=attrgetter("position")):
        terminal_start[terminal.ID] = start
        end += len(terminal.text)
        terminal_end[terminal.ID] = end
        end += 1
        start = end
    tag_to_category = {v: k for k, v in layer1.EdgeTags.__dict__.items() if not k.startswith("__")}
    l1 = p.layer(layer1.LAYER_ID)
    remote_counter = 1
    lines = [l1.heads[0].to_text()]
    units = [unit for unit in l1.all if unit.tag == layer1.NodeTags.Foundational and unit.ftags]
    units = sorted(units, key=attrgetter("start_position", "end_position"))
    unit_to_id = {unit.ID: str(i) for i, unit in enumerate(units, start=1)}
    for unit in units:
        terminals = unit.get_terminals()
        if terminals:
            spans = []
            for terminal in terminals:
                start = terminal_start[terminal.ID]
                end = terminal_end[terminal.ID]
                if not spans or spans[-1][1] < start - 1:
                    spans.append((start, end))
                else:
                    spans[-1] = (spans[-1][0], end)
            lines.append("\t".join(("T" + unit_to_id[unit.ID],
                                    "|".join(tag_to_category.get(tag, tag) for tag in unit.ftags) + " " +
                                    ";".join("%d %d" % (s, e) for s, e in spans),
                                    unit.to_text())))
        for edge in unit.incoming:
            if edge.attrib.get("remote"):
                lines.append("\t".join(("R%d" % remote_counter,
                                        " ".join(("|".join(tag_to_category.get(tag, tag) for tag in edge.tags),
                                                  "parent:T" + unit_to_id[edge.parent.ID],
                                                  "child:T" + unit_to_id[edge.child.ID])))))
                remote_counter += 1
    return "\n".join(lines)



import argparse
import os
from glob import glob

desc = """Replaces the tokens according to a dictionary."""


def read_dictionary_from_file(filename):
    f = open(filename, encoding="utf-8")
    d = {}
    for line in f:
        fields = line.strip().split()
        d[fields[0]] = fields[1]
        d[fields[0].strip().encode('ascii', 'xmlcharrefreplace').decode()] = \
            fields[1].strip().encode('ascii', 'xmlcharrefreplace').decode()
    print(d)
    return d


def main(args):
    os.makedirs(args.out_dir, exist_ok=True)
    replacement_dict = read_dictionary_from_file(args.dict)
    for pattern in args.filenames:
        for filename in sorted(glob(pattern)) or [pattern]:
            basename = os.path.basename(filename)
            with open(os.path.join(args.out_dir, basename), "w", encoding="utf-8") as outfile:
                with open(filename, encoding="utf-8") as infile:
                    xml_string = infile.read()
                for k, v in replacement_dict.items():
                    if args.whole_word:
                        xml_string = xml_string.replace("text=\"" + k + "\"", "text=\"" + v + "\"")
                    else:
                        xml_string = xml_string.replace(k, v)
                print(xml_string, file=outfile, end="")
    print("Done")


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+", help="files to replace tokens in")
    argparser.add_argument("-o", "--out-dir", default=".", help="output directory for changed XMLs")
    argparser.add_argument("-d", "--dict",
                           help="filename to read the dictionary from. the file should have one line per entry, in the"
                                " format of <original text> <replaced text>")
    argparser.add_argument("-w", "--whole-word", action="store_true", help="replace whole word")
    main(argparser.parse_args())



import argparse
import os
import sys
import urllib.request
from itertools import product

from ucca import layer1, convert
from uccaapp.download_task import TaskDownloader

desc = "Get all units according to a specified filter. Units that meet any of the filters are output."

CONSECUTIVE = "CONSECUTIVE"
SUBSEQUENCE = "SUBSEQUENCE"
SUBSET = "SUBSET"


def read_amr_roles(role_type):
    file_name = "have-" + role_type + "-role-91-roles-v1.06.txt"
    if not os.path.exists(file_name):
        url = r"http://amr.isi.edu/download/lists/" + file_name
        try:
            urllib.request.urlretrieve(url, file_name)
        except OSError as e:
            raise IOError("Must download %s and have it in the current directory when running the script" % url) from e
    with open(file_name) as f:
        return [line.split()[1] for line in map(str.strip, f) if line and not line.startswith(("#", "MAYBE"))]


AMR_ROLE = {role for role_type in ("org", "rel") for role in read_amr_roles(role_type)}
TOKEN_CLASSES = {
    "[ROLE]": AMR_ROLE
}


def get_top_level_ancestor(node):
    """
    Traverses the passage upwards until a unit which is immediately below the root is reached
    :param node:
    :return:
    """
    # if node is already the root, return it
    if not node.fparent:
        return node
    parent = node
    while parent.fparent.fparent:
        parent = parent.fparent
    return parent


def tokens_match(unit_tokens, query_tokens, mode):
    """
    :param unit_tokens: candidate unit tokens, as a list of strings
    :param query_tokens: list of lists of tokens to look for, each list representing alternatives for a position
    :param mode: CONSECUTIVE, SUBSET, SUBSEQUENCE
    :return whether query_tokens is contained in unit_tokens
    """
    if mode == SUBSET:
        return any(set(query).issubset(unit_tokens) for query in product(*query_tokens))
    indices = []
    for alternatives in query_tokens:
        index = None
        for alternative in alternatives:
            try:
                index = unit_tokens.index(alternative)
            except ValueError:
                pass
        if index is None:
            return False
        indices.append(index)
    if mode == CONSECUTIVE:
        return indices == list(range(indices[0], indices[-1] + 1))
    elif mode == SUBSEQUENCE:
        return indices == sorted(indices)
    raise ValueError("Invalid option for token mode: " + mode)


## def main(output = None, comment = False, sentence_level = False, categories = (), tokens = (), tokens_mode = CONSECUTIVE,
##          case_insensitive = False, tokens_by_file = False, remotes = False, write = False, **kwargs):
##     if tokens_by_file:
##         with open(tokens[0]) as f:
##             token_lists = [line.strip().split() for line in f]
##     elif tokens != ():
##         token_lists = [tokens]
##     else:
##         token_lists = ()

##     filtered_nodes = []
##     for passage, task_id, user_id in TaskDownloader(**kwargs).download_tasks(write=False, **kwargs):
##         if sentence_level:
##             cur_passages = convert.split2sentences(passage)
##             all_nodes = [p.layer(layer1.LAYER_ID).heads[0] for p in cur_passages]
##         else:
##             all_nodes = list(passage.layer(layer1.LAYER_ID).all)
##         for node in all_nodes:
##             if comment and node.extra.get("remarks"):
##                 filtered_nodes.append(("comment",node,task_id,user_id))
##             if remotes and len([n for n in node.outgoing if n.attrib.get("remote")]) > 0:
##                 filtered_nodes.append(("remotes", node, task_id, user_id))
##             if token_lists and not node.attrib.get("implicit"):
##                 for token_list in token_lists:
##                     unit_tokens = [t.text for t in node.get_terminals(punct=True)]
##                     if case_insensitive:
##                         unit_tokens = [x.lower() for x in unit_tokens]
##                         token_list = [x.lower() for x in token_list]
##                     if tokens_match(unit_tokens, token_list, tokens_mode):
##                         filtered_nodes.append(('TOKENS', node, task_id, user_id))
##             else:
##                 all_tags = [c.tag for edge in node for c in edge.categories]
##                 intersection = set(categories).intersection(all_tags)

def filter_nodes(categories=(), tokens=(), tokens_mode=CONSECUTIVE, case_insensitive=False, comment=False,
                 sentence_level=False, remotes=False, **kwargs):
    for passage, task_id, user_id in TaskDownloader(**kwargs).download_tasks(**kwargs):
        for node in [p.layer(layer1.LAYER_ID).heads[0] for p in convert.split2sentences(passage)] if sentence_level \
                else passage.layer(layer1.LAYER_ID).all:
            if comment and node.extra.get("remarks"):
                yield "comment", node, task_id, user_id
            if remotes:
                if node.attrib.get("implicit"):
                    yield 'IMPLICIT', node, task_id, user_id
                for e in node.incoming:
                    if e.attrib.get("remote"):
                        yield 'REMOTE', e.parent, task_id, user_id
            if tokens and not node.attrib.get("implicit"):
                unit_tokens = [t.text for t in node.get_terminals(punct=True)]
                if case_insensitive:
                    unit_tokens = [x.lower() for x in unit_tokens]
                    tokens = [x.lower() for x in tokens]
                if tokens_match(unit_tokens, tokens, tokens_mode):
                    yield 'TOKENS', node, task_id, user_id
            elif categories:
                intersection = set(categories).intersection(c.tag for e in node for c in e.categories)
                if intersection:
                    yield str(intersection), node, task_id, user_id


def main(output=None, tokens=(), **kwargs):
    kwargs["write"] = False
    f = open(output, 'w', encoding="utf-8") if output else sys.stdout
    expanded_tokens = [TOKEN_CLASSES.get(token, [token]) for token in tokens]
    for filter_type, node, task_id, user_id in filter_nodes(tokens=expanded_tokens, **kwargs):
        ancestor = get_top_level_ancestor(node)
        print(filter_type, task_id, user_id, node.extra.get("tree_id"), node.to_text(),
              ancestor, str(node.extra.get("remarks")).replace("\n", "|"), file=f, sep="\t", flush=True)
    if output:
        f.close()


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    TaskDownloader.add_arguments(argument_parser)
    argument_parser.add_argument("--output", help="output file name")
    argument_parser.add_argument("--categories", nargs="+", default=(),
                                 help="Abbreviations of the names of the categories to filter by")
    argument_parser.add_argument("--tokens", nargs="+", default=(),
                                 help="Tokens to filter by")
    argument_parser.add_argument("--tokens-by-file", action="store_true",
                                 help="tokens will be specified in a file instead of in the command line. Each line consists of space delimited list of tokens.")
    argument_parser.add_argument("--tokens-mode", default=CONSECUTIVE,
                                 help="mode of search for the tokens: CONSECUTIVE,SUBSEQUENCE,SUBSET")
    argument_parser.add_argument("--sentence-level", action="store_true",
                                 help="output sentences rather than units")
    argument_parser.add_argument("--case-insensitive", action="store_true",
                                 help="make tokens search case insensitive")
    argument_parser.add_argument("--comment", action="store_true", help="Output all the units that have comments")
    argument_parser.add_argument("--remotes", action="store_true", help="Output all the units that have remote children")

    main(**vars(argument_parser.parse_args()))




from collections import OrderedDict

import pytest

from ucca import textutil
from ucca.constructions import CATEGORIES_NAME, DEFAULT, CONSTRUCTIONS, extract_candidates
from .conftest import PASSAGES, loaded, loaded_valid, multi_sent, crossing, discontiguous, l1_passage, empty

"""Tests the constructions module functions and classes."""


def assert_spacy_not_loaded(*args, **kwargs):
    del args, kwargs
    assert False, "Should not load spaCy when passage is pre-annotated"


def extract_and_check(p, constructions=None, expected=None):
    d = OrderedDict((construction, [candidate.edge for candidate in candidates]) for construction, candidates in
                    extract_candidates(p, constructions=constructions).items() if candidates)
    if expected is not None:
        hist = {c.name: len(e) for c, e in d.items()}
        assert hist == expected, " != ".join(",".join(sorted(h)) for h in (hist, expected))


@pytest.mark.parametrize("create, expected", (
        (loaded, {'P': 1, 'remote': 1, 'E': 3, 'primary': 15, 'U': 2, 'F': 1, 'C': 3, 'A': 1, 'D': 1, 'L': 2, 'mwe': 2,
                  'H': 5, 'implicit': 1, 'main_rel': 1}),
        (loaded_valid, {'P': 1, 'remote': 1, 'E': 3, 'primary': 15, 'U': 2, 'F': 1, 'C': 3, 'A': 1, 'D': 1, 'L': 2,
                        'mwe': 2, 'H': 5, 'implicit': 1, 'main_rel': 1}),
        (multi_sent, {'U': 4, 'P': 3, 'mwe': 2, 'H': 3, 'primary': 6, 'main_rel': 2}),
        (crossing, {'U': 3, 'P': 2, 'remote': 1, 'mwe': 1, 'H': 2, 'primary': 3, 'main_rel': 2}),
        (discontiguous, {'G': 1, 'U': 2, 'E': 2, 'primary': 13, 'P': 3, 'F': 1, 'C': 1, 'A': 3, 'D': 2,
                         'mwe': 6, 'H': 3, 'implicit':3, 'main_rel': 2}),
        (l1_passage, {'P': 2, 'mwe': 4, 'H': 3, 'primary': 11, 'U': 2, 'A': 5, 'D': 1, 'L': 2, 'remote': 2, 'S': 1,
                      'implicit':1, 'main_rel': 3}),

        (empty, {}),
))
def test_extract_all(create, expected):
    extract_and_check(create(), constructions=CONSTRUCTIONS, expected=expected)


@pytest.mark.parametrize("create", PASSAGES)
@pytest.mark.parametrize("constructions", (DEFAULT, [CATEGORIES_NAME]), ids=("default", CATEGORIES_NAME))
def test_extract(create, constructions, monkeypatch):
    monkeypatch.setattr(textutil, "get_nlp", assert_spacy_not_loaded)
    extract_and_check(create(), constructions=constructions)



#!/usr/bin/env python3

import argparse
import os
import sys
from itertools import count

from ucca.convert import split2paragraphs
from ucca.ioutil import passage2file, get_passages_with_progress_bar, external_write_mode
from ucca.normalization import normalize

desc = """Parses XML files in UCCA standard format, and writes a passage per paragraph."""


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    i = 0
    for passage in get_passages_with_progress_bar(args.filenames, "Splitting"):
        for paragraph in split2paragraphs(
                passage, remarks=args.remarks, lang=args.lang, ids=map(str, count(i)) if args.enumerate else None):
            i += 1
            outfile = os.path.join(args.outdir, args.prefix + paragraph.ID + (".pickle" if args.binary else ".xml"))
            if args.verbose:
                with external_write_mode():
                    print(paragraph, file=sys.stderr)
                    print("Writing passage file for paragraph '%s'..." % outfile, file=sys.stderr)
            if args.normalize:
                normalize(paragraph)
            passage2file(paragraph, outfile, binary=args.binary)


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+", help="passage file names to convert")
    argparser.add_argument("-o", "--outdir", default=".", help="output directory")
    argparser.add_argument("-p", "--prefix", default="", help="output filename prefix")
    argparser.add_argument("-r", "--remarks", action="store_true", help="annotate original IDs")
    argparser.add_argument("-l", "--lang", default="en", help="language two-letter code for paragraph model")
    argparser.add_argument("-b", "--binary", action="store_true", help="write in pickle binary format (.pickle)")
    argparser.add_argument("-e", "--enumerate", action="store_true", help="set output paragraph ID by global order")
    argparser.add_argument("-N", "--no-normalize", dest="normalize", action="store_false",
                           help="do not normalize passages after splitting")
    argparser.add_argument("-v", "--verbose", action="store_true", help="print information about every split paragraph")
    main(argparser.parse_args())



import json
import logging
import os
from time import sleep

import requests

"""
API code for accessing v1.0 of the UCCAApp server
"""

DEFAULT_SERVER = "http://ucca-demo.cs.huji.ac.il"
API_PREFIX = "/api/v1/"
SERVER_ADDRESS_ENV_VAR = "UCCA_APP_SERVER_ADDRESS"
AUTH_TOKEN_ENV_VAR = "UCCA_APP_AUTH_TOKEN"
EMAIL_ENV_VAR = "UCCA_APP_EMAIL"
PASSWORD_ENV_VAR = "UCCA_APP_PASSWORD"
PROJECT_ID_ENV_VAR = "UCCA_APP_PROJECT_ID"
SOURCE_ID_ENV_VAR = "UCCA_APP_SOURCE_ID"
USER_ID_ENV_VAR = "UCCA_APP_USER_ID"
MAX_RETRIES = 3
RETRY_WAIT_DURATION = 60


class ServerAccessor:
    def __init__(self, server_address, email, password, auth_token=None, verbose=False, **kwargs):
        logging.basicConfig(level=logging.DEBUG if verbose else logging.INFO)
        server_address = server_address or os.environ.get(SERVER_ADDRESS_ENV_VAR, DEFAULT_SERVER)
        self.prefix = server_address + API_PREFIX
        self.headers = {}  # Needed for self.request (login)
        try:
            token = auth_token or os.environ.get(AUTH_TOKEN_ENV_VAR) or self.login(
                email or os.environ[EMAIL_ENV_VAR], password or os.environ[PASSWORD_ENV_VAR])["token"]
        except KeyError as e:
            raise ValueError("Must set either --auth-token, or --email and --password."
                             "Alternatively, set the %s environment variable, or %s and %s" %
                             (AUTH_TOKEN_ENV_VAR, EMAIL_ENV_VAR, PASSWORD_ENV_VAR)) from e
        self.headers["Authorization"] = "Token " + token
        self.source = self.project = self.layer = self.user = None

    def set_source(self, source_id=None):
        try:
            self.source = self.get_source(int(os.environ[SOURCE_ID_ENV_VAR]) if source_id is None else source_id)
        except KeyError as e:
            raise ValueError("Must set --source-id or the %s environment variable" % SOURCE_ID_ENV_VAR) from e

    def set_project(self, project_id=None):
        try:
            self.project = self.get_project(int(os.environ[PROJECT_ID_ENV_VAR]) if project_id is None else project_id)
        except KeyError as e:
            raise ValueError("Must set --project-id or the %s environment variable" % PROJECT_ID_ENV_VAR) from e
        self.layer = self.get_layer(self.project["layer"]["id"])

    def set_user(self, user_id=None):
        try:
            self.user = dict(id=int(os.environ[USER_ID_ENV_VAR]) if user_id is None else user_id)
        except KeyError as e:
            raise ValueError("Must set --user-id or the %s environment variable" % USER_ID_ENV_VAR) from e

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("--server-address", help="UCCA-App server, otherwise set by " + SERVER_ADDRESS_ENV_VAR)
        argparser.add_argument("--email", help="UCCA-App email, otherwise set by " + EMAIL_ENV_VAR)
        argparser.add_argument("--password", help="UCCA-App password, otherwise set by " + PASSWORD_ENV_VAR)
        argparser.add_argument("--auth-token", help="authorization token (required only if email or password missing), "
                                                    "otherwise set by " + AUTH_TOKEN_ENV_VAR)
        argparser.add_argument("-v", "--verbose", action="store_true", help="detailed output")

    @staticmethod
    def add_source_id_argument(argparser):
        argparser.add_argument("--source-id", type=int, help="source id, otherwise set by " + SOURCE_ID_ENV_VAR)

    @staticmethod
    def add_project_id_argument(argparser):
        argparser.add_argument("--project-id", type=int, help="project id, otherwise set by " + PROJECT_ID_ENV_VAR)

    @staticmethod
    def add_user_id_argument(argparser):
        argparser.add_argument("--user-id", type=int, help="user id, otherwise set by " + USER_ID_ENV_VAR)

    def request(self, method, url_suffix, **kwargs):
        response = None
        for _ in range(MAX_RETRIES):
            response = requests.request(method, self.prefix + str(url_suffix), headers=self.headers, **kwargs)
            if response.status_code != 500:
                break
            sleep(RETRY_WAIT_DURATION)

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise requests.exceptions.HTTPError(response.text) from e
        return response

    def login(self, email, password):
        return self.request("post", "login", json=dict(email=email, password=password)).json()

    @staticmethod
    def type(data):
        return data.get("type", "").lower()

    def update(self, data, prefix):
        logging.debug("Updating %s %s: %s" % (self.type(data), prefix, json.dumps(data)))
        out = self.request("put", prefix + "/%s/" % data["id"], json=data).json()
        logging.debug("Updated %s %s: %s" % (data.get("type", ""), prefix, json.dumps(out)))
        return out

    def create(self, data, prefix):
        logging.debug("Creating %s %s: %s" % (self.type(data), prefix, json.dumps(data)))
        out = self.request("post", prefix + "/", json=data).json()
        logging.debug("Created %s %s: %s" % (self.type(data), prefix, json.dumps(out)))
        return out

    def get(self, _id, prefix):
        logging.debug("Getting %s %s" % (prefix, _id))
        out = self.request("get", "%s/%s" % (prefix, _id)).json()
        logging.debug("Got %s: %s" % (prefix, json.dumps(out)))
        return out

    def submit_task(self, submit=True, **kwargs):
        logging.debug("Submitting %s task: %s" % (self.type(kwargs), json.dumps(kwargs)))
        out = None
        if self.type(kwargs) == "annotation" or not submit:  # Annotation tasks require drafting before submission
            out = self.request("put", "user_tasks/%s/draft" % kwargs["id"], json=kwargs).json()
            logging.debug("Drafted %s task: %s" % (self.type(kwargs), json.dumps(out)))
        if submit:
            out = self.request("put", "user_tasks/%s/submit" % kwargs["id"], json=kwargs).json()
            logging.debug("Submitted %s task: %s" % (self.type(kwargs), json.dumps(out)))
        return out

    def get_source(self, source_id):
        return self.get(source_id, prefix="sources")

    def get_project(self, project_id):
        return self.get(project_id, prefix="projects")

    def get_layer(self, layer_id):
        return self.get(layer_id, prefix="layers")

    def get_category(self, category_id):
        return self.get(category_id, prefix="categories")

    def create_category(self, **kwargs):
        return self.create(kwargs, prefix="categories")

    def get_user(self, user_id):
        return self.get(user_id, prefix="users")

    def get_task(self, task_id):
        return self.get(task_id, prefix="tasks")

    def create_task(self, **kwargs):
        return self.create(kwargs, prefix="tasks")

    def update_task(self, **kwargs):
        return self.update(kwargs, prefix="tasks")

    def get_user_task(self, task_id):
        return self.get(task_id, prefix="user_tasks")

    def get_passage(self, passage_id):
        return self.get(passage_id, prefix="passages")

    def create_passage(self, **kwargs):
        return self.create(kwargs, prefix="passages")

    def update_passage(self, **kwargs):
        return self.update(kwargs, prefix="passages")



import argparse
import os
import sys
import urllib.request
from itertools import product

from ucca import layer0
from uccaapp.download_task import TaskDownloader


## def main(output = None, comment = False, sentence_level = False, categories = (), tokens = (), tokens_mode = CONSECUTIVE,
##          case_insensitive = False, tokens_by_file = False, remotes = False, write = False, **kwargs):
##     if tokens_by_file:
##         with open(tokens[0]) as f:
##             token_lists = [line.strip().split() for line in f]
##     elif tokens != ():
##         token_lists = [tokens]
##     else:
##         token_lists = ()

##     filtered_nodes = []
##     for passage, task_id, user_id in TaskDownloader(**kwargs).download_tasks(write=False, **kwargs):
##         if sentence_level:
##             cur_passages = convert.split2sentences(passage)
##             all_nodes = [p.layer(layer1.LAYER_ID).heads[0] for p in cur_passages]
##         else:
##             all_nodes = list(passage.layer(layer1.LAYER_ID).all)
##         for node in all_nodes:
##             if comment and node.extra.get("remarks"):
##                 filtered_nodes.append(("comment",node,task_id,user_id))
##             if remotes and len([n for n in node.outgoing if n.attrib.get("remote")]) > 0:
##                 filtered_nodes.append(("remotes", node, task_id, user_id))
##             if token_lists and not node.attrib.get("implicit"):
##                 for token_list in token_lists:
##                     unit_tokens = [t.text for t in node.get_terminals(punct=True)]
##                     if case_insensitive:
##                         unit_tokens = [x.lower() for x in unit_tokens]
##                         token_list = [x.lower() for x in token_list]
##                     if tokens_match(unit_tokens, token_list, tokens_mode):
##                         filtered_nodes.append(('TOKENS', node, task_id, user_id))
##             else:
##                 all_tags = [c.tag for edge in node for c in edge.categories]
##                 intersection = set(categories).intersection(all_tags)

def count_tokens(**kwargs):
    output = []
    for passage, task_id, user_id in TaskDownloader(**kwargs).download_tasks(**kwargs):
        num_tokens = len(passage.layer(layer0.LAYER_ID).all)
        output.append((num_tokens,task_id,user_id))
    return output

def main(output=None, tokens=(), **kwargs):
    kwargs["write"] = False
    f = open(output, 'w', encoding="utf-8") if output else sys.stdout
    for num_tokens, task_id, user_id in count_tokens(**kwargs):
        print(str(num_tokens), task_id, user_id, file=f, sep="\t", flush=True)
    if output:
        f.close()


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser()
    TaskDownloader.add_arguments(argument_parser)
    argument_parser.add_argument("--output", help="output file name")
    main(**vars(argument_parser.parse_args()))




#!/usr/bin/env python3
"""The evaluation script for UCCA layer 1."""
from argparse import ArgumentParser
from itertools import repeat

from ucca import evaluation, constructions, ioutil


def main(args):
    guessed, ref, ref_yield_tags = [None if x is None else ioutil.read_files_and_dirs((x,))
                                    for x in (args.guessed, args.ref, args.ref_yield_tags)]
    if args.match_by_id:
        guessed = match_by_id(guessed, ref)
        ref_yield_tags = match_by_id(ref_yield_tags, ref)
    results = []
    eval_type = evaluation.UNLABELED if args.unlabeled else evaluation.LABELED
    verbose = args.verbose or len(guessed) == 1
    for g, r, ryt in zip(guessed, ref, ref_yield_tags or repeat(None)):
        if len(guessed) > 1:
            print("Evaluating %s%s" % (g.ID, ":" if args.verbose else "..."), end="\r", flush=True)
        if args.verbose:
            print()
        result = evaluation.evaluate(g, r, constructions=args.constructions, units=args.units, fscore=args.fscore,
                                     errors=args.errors, verbose=verbose,
                                     normalize=args.normalize, ref_yield_tags=ryt,
                                     eval_type=evaluation.UNLABELED if args.unlabeled else None)
        if verbose:
            if args.errors:
                result.print_confusion_matrix(as_table=args.as_table)
            if not args.quiet:
                print_f1(result, eval_type)
        results.append(result)
    summarize(args, results, eval_type=eval_type)


def match_by_id(guessed, ref):
    if guessed is None:
        return None
    if len(guessed) != len(ref):
        raise ValueError("Number of passages to compare does not match: %d != %d" % (len(guessed), len(ref)))
    if len(guessed) > 1:
        guessed_by_id = {}
        for g in guessed:
            print("Reading %s..." % g.ID, end="\r", flush=True)
            guessed_by_id[g.ID] = g
        ids = [p.ID for p in ref]
        try:
            return [guessed_by_id[i] for i in ids]
        except KeyError as e:
            raise ValueError("Passage IDs do not match") from e
    return guessed


def print_f1(result, eval_type):
    print("Average %s F1 score: %.3f" % (eval_type, result.average_f1(eval_type)))


def summarize(args, results, eval_type):
    summary = evaluation.Scores.aggregate(results)
    if len(results) > 1:
        if args.verbose:
            print("Aggregated scores:")
        else:
            if not args.quiet:
                if args.fscore:
                    summary.print()
                elif args.errors:
                    print(args.guessed, args.ref)
                    summary.print_confusion_matrix(as_table=args.as_table)
        if not args.quiet:
            print_f1(summary, eval_type=eval_type)
    if args.out_file:
        with open(args.out_file, "w", encoding="utf-8") as f:
            print(*summary.titles(eval_type=eval_type), sep=",", file=f)
            for result in results:
                print(*result.fields(eval_type=eval_type), sep=",", file=f)
        print("Wrote '%s'" % args.out_file)
    for filename, counts in ((args.summary_file, False), (args.counts_file, True)):
        if filename:
            with open(filename, "w", encoding="utf-8") as f:
                print(*summary.titles(eval_type=eval_type, counts=counts), sep=",", file=f)
                print(*summary.fields(eval_type=eval_type, counts=counts), sep=",", file=f)
            print("Wrote '%s'" % filename)
    if args.errors_file:
        with open(args.errors_file, "w", encoding="utf-8") as f:
            summary.print_confusion_matrix(sep=",", as_table=args.as_table, file=f)
        print("Wrote '%s'" % args.errors_file)


def check_args(args):
    if args.out_file or args.summary_file or not (args.units or args.fscore or args.errors):
        args.fscore = True
    if args.errors_file:
        args.errors = True
    return args


if __name__ == "__main__":
    argparser = ArgumentParser(description="Compare two UCCA passages or two directories containing passage files.")
    argparser.add_argument("guessed", help="xml/pickle file name for the guessed annotation, or directory of files")
    argparser.add_argument("ref", help="xml/pickle file name for the reference annotation, or directory of files")
    argparser.add_argument("-r", "--ref-yield-tags", help="xml/pickle file name for reference used for extracting edge "
                                                          "categories for fine-grained annotation "
                                                          "(--constructions categories), or directory of files")
    argparser.add_argument("-u", "--units", action="store_true",
                           help="the units the annotations have in common, and those each has separately")
    argparser.add_argument("-f", "--fscore", action="store_true",
                           help="outputs the traditional P,R,F instead of the scene structure evaluation")
    argparser.add_argument("-e", "--errors", action="store_true",
                           help="prints the error distribution according to its frequency")
    argparser.add_argument("-N", "--no-normalize", dest="normalize", action="store_false",
                           help="do not normalize passages before evaluation")
    argparser.add_argument("-I", "--no-match-by-id", dest="match_by_id", action="store_false",
                           help="do not match passages by ID, instead matching by order")
    argparser.add_argument("--as-table", action="store_true", help="if given --errors, print confusion matrix as table")
    argparser.add_argument("--unlabeled", action="store_true", help="only unlabeled evaluation")
    argparser.add_argument("--out-file", help="file to write results for each evaluated passage to, in CSV format")
    argparser.add_argument("--summary-file", help="file to write aggregated scores to, in CSV format")
    argparser.add_argument("--counts-file", help="file to write aggregated counts to, in CSV format")
    argparser.add_argument("--errors-file", help="file to write aggregated confusion matrix to, in CSV format")
    group = argparser.add_mutually_exclusive_group()
    group.add_argument("-v", "--verbose", action="store_true",
                       help="prints the results for every single pair (always true if there is only one pair)")
    group.add_argument("-q", "--quiet", action="store_true", help="do not print anything")
    constructions.add_argument(argparser)
    main(check_args(argparser.parse_args()))



#!/usr/bin/env python3
import argparse
import logging
import sys

from requests.exceptions import HTTPError
import json

from ucca.convert import to_json, to_text
from ucca.ioutil import get_passages_with_progress_bar
from uccaapp.api import ServerAccessor

try:
    from simplejson.scanner import JSONDecodeError
except ImportError:
    from json.decoder import JSONDecodeError

desc = """Convert a passage file to JSON format and upload to UCCA-App as a completed task"""

# https://github.com/omriabnd/UCCA-App/blob/master/UCCAApp_REST_API_Reference.pdf
# ucca-demo.cs.huji.ac.il or ucca.staging.cs.huji.ac.il
# upload the parse as a (completed) task:
# 0. decide which project and user you want to assign it to
# 1. POST passage (easy format)
# 2. POST task x (of type tokenization)
# 3. PUT task x (submit)
# 4. POST task y (of type annotation with parent x; this is the more complicated format)
# 5. PUT task y (submit)


class TaskUploader(ServerAccessor):
    def __init__(self, user_id, source_id, project_id, **kwargs):
        super().__init__(**kwargs)
        self.set_source(source_id)
        self.set_project(project_id)
        self.set_user(user_id)
        
    def upload_tasks(self, filenames, log=None, submit=True, existing_ids=None, **kwargs):
        del kwargs
        log_h = open(log, "w", encoding="utf-8") if log else None
        if existing_ids:
            with open(existing_ids, "r", encoding="utf-8") as ids_h:
                ids = {old_passage_id: (passage_id, tok_id, ann_id)
                       for (old_passage_id, passage_id, tok_id, ann_id)
                       in map(str.split, ids_h)}
        else:
            ids = None
        try:
            for passage in get_passages_with_progress_bar(filenames, desc="Uploading"):
                logging.debug("Uploading passage %s" % passage.ID)
                task = self.upload_task(passage, log=log_h, submit=submit, ids=ids)
                logging.debug("Submitted task %d" % task["id"])
                yield task
        except HTTPError as e:
            try:
                raise ValueError((e.response.json() if e.response else json.loads(e.args[0]))["detail"]) from e
            except JSONDecodeError:
                raise ValueError(e.response.text) from e
            finally:
                if log:
                    log_h.close()

    def upload_task(self, passage, log=None, submit=True, ids=None, upload=True):
        if ids:
            passage_id, tok_id, ann_id = ids[passage.ID]
            passage_out = self.get_passage(passage_id)
            tok_user_task_out = tok_task_out = self.get_user_task(tok_id)
            ann_user_task_in = self.get_user_task(ann_id)
        else:
            passage_out = self.create_passage(text=to_text(passage, sentences=False)[0], type="PUBLIC",
                                              source=self.source, external_id=passage.ID) if upload else passage
            task_in = dict(type="TOKENIZATION", status="ONGOING", project=self.project, user=self.user,
                           passage=passage_out, manager_comment=passage.ID, user_comment=passage.ID, parent=None,
                           is_demo=False, is_active=True)
            tok_task_out = self.create_task(**task_in) if upload else task_in
            tok_user_task_in = dict(tok_task_out)
            tok_user_task_in.update(to_json(passage, return_dict=True, tok_task=True))
            tok_user_task_out = self.submit_task(**tok_user_task_in) if upload else tok_user_task_in
            task_in.update(parent=tok_task_out, type="ANNOTATION")
            ann_user_task_in = self.create_task(**task_in) if upload else task_in
        ann_user_task_in.update(
            to_json(passage, return_dict=True, tok_task=tok_user_task_out, all_categories=self.layer["categories"]))
        ann_user_task_out = self.submit_task(**ann_user_task_in, submit=submit) if upload else ann_user_task_in
        if log:
            print(passage.ID, passage_out["id"], tok_task_out["id"], ann_user_task_out["id"],
                  file=log, sep="\t", flush=True)
        return ann_user_task_out

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("filenames", nargs="+", help="passage file names to convert and upload")
        argparser.add_argument("-l", "--log", help="filename to write log of uploaded passages to")
        argparser.add_argument("--no-submit", action="store_false", dest="submit", help="do not submit annotation task")
        argparser.add_argument("--existing-ids", help="use existing task IDs from file (output of --log); no creation")
        argparser.add_argument("-n", "--no-upload", action="store_false", dest="upload", help="do not upload anything")
        ServerAccessor.add_project_id_argument(argparser)
        ServerAccessor.add_source_id_argument(argparser)
        ServerAccessor.add_user_id_argument(argparser)
        ServerAccessor.add_arguments(argparser)


def main(**kwargs):
    list(TaskUploader(**kwargs).upload_tasks(**kwargs))


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    TaskUploader.add_arguments(argument_parser)
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)



#! /usr/bin/python3

import argparse
import pickle
from xml.etree.ElementTree import ElementTree, fromstring

import psycopg2

import ucca.convert

desc = """Parses an XML in UCCA site format.

The input can be given as either an XML file or a DB file with passage ID
and user name, and the output is either the standard format XML or
a pickled object.
Possible input methods are using a DB file with pid and user, which gets the
annotation of the specified user for the specified passage from teh DB file,
or using filename of a site-formatted XML file.

"""


def site2passage(filename):
    """Opens a file and returns its parsed Passage object"""
    with open(filename, encoding="utf-8") as f:
        etree = ElementTree().parse(f)
    return ucca.convert.from_site(etree)


def db2passage(handle, pid, user):
    """Gets the annotation of user to pid from the DB handle - returns a passage"""
    handle.execute("SET search_path to oabend")
    handle.execute("SELECT id FROM users WHERE username=%s", (user,))
    uid = handle.fetchone()[0]
    handle.execute("SELECT xml,ts FROM xmls WHERE paid=%s AND uid=%s " +
                   "ORDER BY ts DESC", (pid, uid))
    raw_xml, ts = handle.fetchone()
    #print('extracted passage from '+str(ts))
    return ucca.convert.from_site(fromstring(raw_xml))


def main(args):
    # Checking for illegal combinations
    if args.db and args.filename:
        argparser.error("Only one source, XML or DB file, can be used")
    if (not args.db) and (not args.filename):
        argparser.error("Must specify one source, XML or DB file")
    if args.db and not (args.pid and args.user):
        argparser.error("Must specify a username and a passage ID when " +
                        "using DB file option")
    if (args.pid or args.user) and not args.db:
        argparser.error("Cannot use user and passage ID options without DB file")

    if args.filename:
        passage = site2passage(args.filename)
    else:
        conn = psycopg2.connect(host=args.host, database=args.db)
        c = conn.cursor()
        passage = db2passage(c, args.pid, args.user)

    if args.binary:
        with open(args.binary, "wb") as binf:
            pickle.dump(passage, binf)
    else:
        output = ucca.convert.to_text(passage, lang=args.lang)
        if args.outfile:
            with open(args.outfile, "w", encoding="utf-8") as outf:
                outf.write(output)
        else:
            print(output)


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filename", nargs="?", help="XML file name to convert")
    argparser.add_argument("-o", "--outfile", help="output file for standard XML")
    argparser.add_argument("-b", "--binary", help="output file for binary pickel")
    argparser.add_argument("-d", "--db", help="DB file to get input from")
    argparser.add_argument("--host", help="DB host server to get input from")
    argparser.add_argument("-p", "--pid", type=int, help="PassageID to query DB")
    argparser.add_argument("-u", "--user", help="Username to DB query")
    argparser.add_argument("-l", "--lang", default="en", help="language two-letter code for sentence model")
    main(argparser.parse_args())



import csv
import os
from argparse import ArgumentParser

from ucca import layer0, layer1
from ucca.ioutil import get_passages_with_progress_bar, write_passage
from ucca.normalization import fparent, copy_edge, traverse_up_centers

desc = """Change articles to Function, complying with UCCA v2 guidelines"""

ARTICLES = {
    "de": ("der", "die", "das", "den", "dem", "des", "ein", "eine", "einen", "einem", "eines"),
    "en": ("a", "an", "the"),
}

REFLEXIVES = {
    "en": ("herself", "himself", "itself", "themselves", "yourself", "yourselves", "myself", "ourselves", "oneself"),
}

NONE = {
    "de": ("kein", "keine", "keinen", "keines", "keiner", "keinem"),
}


def change_article_to_function(terminal, parent, lang):
    if terminal.text.lower() in ARTICLES[lang]:
        for edge in parent.incoming:
            if not edge.attrib.get("remote"):
                # First, remove Functions to avoid duplicates
                if len(edge.categories) > 1:
                    edge.categories = [category for category in edge.categories
                                       if category.tag != layer1.EdgeTags.Function]
                # Then replace Elaborators to Functions
                for category in edge.categories:
                    if category.tag == layer1.EdgeTags.Elaborator:
                        category.tag = layer1.EdgeTags.Function
                        return True


def insert_reflexive_into_relation(terminal, parent, lang):
    if terminal.text.lower() in REFLEXIVES.get(lang, ()):
        for edge in parent.incoming:
            if not edge.attrib.get("remote"):
                for category in edge.categories:
                    if category.tag == layer1.EdgeTags.Adverbial:
                        for grandparent in parent.parents:
                            new_parent = grandparent.process or grandparent.state
                            if new_parent is not None:
                                while any(layer1.EdgeTags.Center in e.tags for e in new_parent):
                                    new_parent = next(e for e in new_parent if layer1.EdgeTags.Center in e.tags).child
                                parent.destroy()
                                new_parent.add(layer1.EdgeTags.Terminal, terminal)
                                return True


def change_none_to_quantifier(terminal, parent, lang):
    if terminal.text.lower() in NONE.get(lang, ()):
        parent = traverse_up_centers(parent)
        for edge in parent.incoming:
            if not edge.attrib.get("remote"):
                for category in edge.categories:
                    if category.tag == layer1.EdgeTags.Adverbial:
                        for participant_edge in edge.parent:
                            if layer1.EdgeTags.Participant in participant_edge.tags:
                                new_parent = participant_edge.child
                                if new_parent.start_position == terminal.position + 1:
                                    if not new_parent.centers:
                                        edges = new_parent.outgoing
                                        center = new_parent.layer.add_fnode(new_parent, layer1.EdgeTags.Center)
                                        for sub_edge in edges:
                                            copy_edge(sub_edge, center)
                                            new_parent.remove(sub_edge)
                                    category.tag = layer1.EdgeTags.Quantifier
                                    participant_edge.add(layer1.EdgeTags.Adverbial)
                                    copy_edge(edge, new_parent)
                                    edge.parent.remove(edge)
                                    return True


RULES = (change_article_to_function, insert_reflexive_into_relation, change_none_to_quantifier)


def convert_passage(passage, lang, report_writer):
    for rule in RULES:
        for terminal in passage.layer(layer0.LAYER_ID).all:
            parent = fparent(terminal)
            if len(parent.children) == 1 and rule(terminal, parent, lang):
                report_writer.writerow((rule.__name__, passage.ID, terminal.ID, parent, fparent(terminal)))


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    with open(args.outfile, "w", encoding="utf-8", newline="") as f:
        writer = csv.writer(f)
        writer.writerow(("rule", "passage", "terminal", "before", "after"))
        for passage in get_passages_with_progress_bar(args.passages, desc="Converting"):
            convert_passage(passage, lang=passage.attrib.get("lang", args.lang), report_writer=writer)
            write_passage(passage, outdir=args.outdir, prefix=args.prefix, verbose=args.verbose)
            f.flush()
    print("Wrote '%s'" % args.outfile)


if __name__ == "__main__":
    argparser = ArgumentParser(description=desc)
    argparser.add_argument("passages", nargs="+", help="the corpus, given as xml/pickle file names")
    argparser.add_argument("-l", "--lang", choices=ARTICLES, help="two-letter language code for article list")
    argparser.add_argument("-o", "--outdir", default=".", help="output directory")
    argparser.add_argument("-p", "--prefix", default="", help="output filename prefix")
    argparser.add_argument("-O", "--outfile", default=os.path.splitext(argparser.prog)[0] + ".csv", help="log file")
    argparser.add_argument("-v", "--verbose", action="store_true", help="print tagged text for each passage")
    main(argparser.parse_args())
